package httpUtil

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"time"
)

// Request 封装的 POST 请求工具类
type Request struct {
	client *http.Client
}

// NewRequest 创建一个新的 PostRequest 实例
func NewRequest() *Request {
	return &Request{
		client: &http.Client{
			Timeout: 10 * time.Second, // 设置请求超时时间
		},
	}
}

// SendJSON 发送一个 JSON 格式的 POST 请求
func (p *Request) SendJSON(urlPath string, headers map[string]string, body interface{}) (string, error) {
	// 将请求体转为 JSON
	requestBody, err := json.Marshal(body)
	if err != nil {
		return "", fmt.Errorf("无法编码 JSON 请求体: %v", err)
	}

	// 创建 HTTP 请求
	req, err := http.NewRequest("POST", urlPath, bytes.NewBuffer(requestBody))
	if err != nil {
		return "", fmt.Errorf("无法创建请求: %v", err)
	}

	// 设置请求头
	req.Header.Set("Content-Type", "application/json")
	for key, value := range headers {
		req.Header.Set(key, value)
	}

	// 发送请求
	resp, err := p.client.Do(req)
	if err != nil {
		return "", fmt.Errorf("发送请求失败: %v", err)
	}
	defer resp.Body.Close()

	// 读取响应内容
	respBody, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return "", fmt.Errorf("读取响应体失败: %v", err)
	}

	// 返回响应内容
	return string(respBody), nil
}

// SendFormData 发送一个 form-data 格式的 POST 请求
func (p *Request) SendFormData(urlPath string, headers map[string]string, params map[string]string) (string, error) {
	// 将参数拼接为表单数据
	formData := url.Values{}
	for key, value := range params {
		formData.Set(key, value)
	}

	// 创建 HTTP 请求
	req, err := http.NewRequest("POST", urlPath, strings.NewReader(formData.Encode()))
	if err != nil {
		return "", fmt.Errorf("无法创建请求: %v", err)
	}

	// 设置请求头
	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")
	for key, value := range headers {
		req.Header.Set(key, value)
	}

	// 发送请求
	resp, err := p.client.Do(req)
	if err != nil {
		return "", fmt.Errorf("发送请求失败: %v", err)
	}
	defer resp.Body.Close()

	// 读取响应内容
	respBody, err := io.ReadAll(resp.Body)
	if err != nil {
		return "", fmt.Errorf("读取响应体失败: %v", err)
	}

	// 返回响应内容
	return string(respBody), nil
}

// SendGET 发送一个 GET 请求
func (p *Request) SendGET(urlPath string, headers map[string]string, params map[string]string) (string, error) {
	// 如果有查询参数，拼接到 URL 中
	if len(params) > 0 {
		urlValues := url.Values{}
		for key, value := range params {
			urlValues.Set(key, value)
		}
		urlPath = fmt.Sprintf("%s?%s", urlPath, urlValues.Encode())
	}

	// 创建 HTTP GET 请求
	req, err := http.NewRequest("GET", urlPath, nil)
	if err != nil {
		return "", fmt.Errorf("无法创建请求: %v", err)
	}

	// 设置请求头
	for key, value := range headers {
		req.Header.Set(key, value)
	}

	// 发送请求
	resp, err := p.client.Do(req)
	if err != nil {
		return "", fmt.Errorf("发送请求失败: %v", err)
	}
	defer resp.Body.Close()

	// 读取响应内容
	respBody, err := io.ReadAll(resp.Body)
	if err != nil {
		return "", fmt.Errorf("读取响应体失败: %v", err)
	}

	// 返回响应内容
	return string(respBody), nil
}
